@@ -51,7 +51,7 @@
 #include "utils.h"
 #include "waitanypid.h"
 
-static const char *SCCSid = "@(#)cygrunsrv V1.63, " __DATE__ "\n";
+static const char *SCCSid = "@(#)cygrunsrv V1.64, " __DATE__ "\n";
 
 struct option longopts[] = {
   { "install", required_argument, NULL, 'I' },
@@ -1506,16 +1506,31 @@ list_services (const char *server, bool verbose)
         }
       
       if (!QueryServiceConfig (sh, qsc_buf, QSC_BUF_SIZE, &bytes_needed))
-        fprintf (stderr, "%s: warning: QueryServiceConfig failed for '%s': "
-                 "Win32 error %u\n%s", appname, srv_buf[i].lpServiceName,
-                 (unsigned) GetLastError (), winerror (GetLastError ()));
+	{
+	  fprintf (stderr, "%s: warning: QueryServiceConfig failed for '%s': "
+		   "Win32 error %u\n%s", appname, srv_buf[i].lpServiceName,
+		   (unsigned) GetLastError (), winerror (GetLastError ()));
+	  CloseServiceHandle (sh);
+	  sh = (SC_HANDLE) 0;
+	  continue;
+	}
+      /* The path might be quoted.  Drop the quotes. */
+      char service_path[strlen (qsc_buf->lpBinaryPathName) + 1];
+      char *endptr = stpcpy (service_path, qsc_buf->lpBinaryPathName[0] == '"'
+					   ? qsc_buf->lpBinaryPathName + 1
+					   : qsc_buf->lpBinaryPathName);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
+      if (endptr > service_path && endptr[-1] == '"')
+	endptr[-1] = '\0';
+#pragma GCC diagnostic pop
 
       /* is this us? */
-      else if (same_filename (qsc_buf->lpBinaryPathName, mypath))
-        {          
+      if (same_filename (service_path, mypath))
+        {
           if (!verbose)
 	    {
-	      if (strcasecmp (qsc_buf->lpBinaryPathName, mypath))
+	      if (strcasecmp (service_path, mypath))
 		printf ("(%s)\n", srv_buf[i].lpServiceName);
 	      else
 		printf ("%s\n", srv_buf[i].lpServiceName);
